{% extends "base.html" %}
{% block content %}
<h2>Dashboard</h2>
<div id="dashboard-content"
     hx-get="/fragment/dashboard"
     hx-trigger="load, every 5s"
     hx-swap="innerHTML">
<p class="htmx-indicator-inline" aria-busy="true">Loading from cache…</p>
</div>
{% endblock %}
