{% if issues %}
<figure>
<table class="sortable">
    <thead>
        <tr><th>ID</th><th>Title</th><th>Author</th><th>Labels</th><th>Comments</th><th>Status</th></tr>
    </thead>
    <tbody>
        {% for i in issues %}
        {% set _mine_users = (state.usernames | default({})).values() | list %}
        {% set _involved = i.author in _mine_users or (i.assignees | default([]) | select("in", _mine_users) | list | length > 0) or (i.participants | default([]) | select("in", _mine_users) | list | length > 0) %}
        <tr class="{% if state and _involved %}row-mine{% endif %}">
            <td>#{{ i.iid }}</td>
            <td><a href="{{ i.web_url }}" target="_blank">{% if i.confidential %}&#x1F512; {% endif %}{{ i.title }}</a></td>
            <td>{{ i.author }}</td>
            <td>{% for l in (i.labels | default([])) %}<span class="label-badge">{{ l }}</span> {% endfor %}</td>
            <td>{{ i.user_notes_count or "" }}</td>
            <td><span class="badge-{{ i.state }}">{{ i.state }}</span></td>
        </tr>
        {% endfor %}
    </tbody>
</table>
</figure>
{% else %}
<p>No open issues.</p>
{% endif %}
